/*
 * Speak Recorder
 * Copyright  2003 by Schou Industries ApS.
 */

#include <iostream>
#include <unistd.h>
#include <termios.h>
#ifdef HAS_CONFIG_H
#include "config.h"
#endif

#include "speakrec.h"
#include "blind.h"

#define BUTTONY 4
#define SEGMENTY 5
#define VOLUMEY 6

/*
struct {
	char *name;
	button_name but;
} bbuttons[] = {
	{ name:"Record", but:but_record  },
	{ name:"reWind", but:but_rewind  },
	{ name:"Forwrd", but:but_forward },
	{ name:" Stop ", but:but_stop    },
	{ name:" Play ", but:but_play    },
	{ name:" Edit ", but:but_edit    },
	{ name:" nexT ", but:but_track_next }
};
*/

BlindConsole::BlindConsole(void) {
	struct termios termattr;

	nameofdevice = "Terminal, blind mode TTY";
	fd = fileno(stdin);
	if (0 > tcgetattr(fd, &save_termios))
		fprintf(stderr,"Error save_termios\n");
	memcpy(&termattr, &save_termios, sizeof(termattr));
	termattr.c_lflag &= ~(ICANON | ECHO);
	termattr.c_cc[VMIN]  = 1;
	termattr.c_cc[VTIME] = 0;
	if (0 > tcsetattr(fd, TCSANOW, &termattr)) 
		fprintf(stderr,"Error set termattr\n");
	redraw();
}

BlindConsole::~BlindConsole(void) {
	tcsetattr(fd, TCSANOW, &save_termios);
	cout << "\e[0m" << endl;
}

int BlindConsole::flush() {
	cout << "\e[" << VOLUMEY << ";" << currentpeak << "H";
	fflush(stdout);
}

struct {
	char key;
	tape_key cmd;
	char *name;
} blind_key[] = {
	{ key:'s', cmd:key_stop,      "Stop" },
	{ key:'p', cmd:key_play,      "Play" },
	{ key:'r', cmd:key_record,    "Record" },
	{ key:'t', cmd:key_track_next,"nexT track" },
	{ key:'v', cmd:key_track_prev,"preV track" },
	{ key:'w', cmd:key_rewind_inc,"reWind" },
	{ key:'W', cmd:key_rewind_dec, NULL },
	{ key:'f', cmd:key_forward_inc, "Forward" },
	{ key:'F', cmd:key_forward_dec, NULL },
	{ key:'m', cmd:key_menu, "Menu" },
	{ key:'q', cmd:key_quit, "Quit" }
};

#define SIZE_BLIND_KEY (sizeof(blind_key) / sizeof(blind_key[0]))

tape_key BlindConsole::getcommand(fd_set *read_set, tape_key lastcmd) {
	char ch;
	int i = 0;

	if (FD_ISSET(fd, read_set)) {
		ch = getc(stdin);
		while (i<SIZE_BLIND_KEY && blind_key[i].key != ch)
			i++;
		if (i>=SIZE_BLIND_KEY) {
			return key_none;
		} else {
			return blind_key[i].cmd;
		}
	}

	return key_none;
}

int BlindConsole::fd_set_read(fd_set *read_set) {
	FD_SET(fd, read_set);

	return 0;
}

int BlindConsole::fd_set_write(fd_set *write_set) {

	return 0;
}

int BlindConsole::redraw(void) {
	cout << "\e[2J\e[H";
	return 0;
}

int BlindConsole::setvolume(int newvol, int peakvol) {
	int i = 0;

	newvol /= 2;
	peakvol /= 2;

	cout << "\e[" << VOLUMEY << "H";
	cout << "\e[K"; /* Clear to end of line */
	while (i++<newvol)
		cout << "*";
	cout << "\e[" << VOLUMEY << ";" << peakvol << "H";
	cout << "+";
	currentpeak = peakvol;

	return 0;
}

int BlindConsole::showsegment(int num, unsigned char value) {
	printf("\e[%d;%dH", SEGMENTY, num+1 );
	printf("%c", value & 0x7F);

	return 0;
}

struct {
	button_name but;
	char *name;
} blindbutname[] = {
	{ but_none,       NULL      },
	{ but_stop,       "Stop"    },
	{ but_play,       "Play"    },
	{ but_record,     "Record"  },
	{ but_track_next, "nexT"    },
	{ but_rewind,     "Rewind"  },
	{ but_forward,    "Forward" },
	{ but_edit,       "Edit"    }
};

#define SIZE_BLINDBUTNAME (sizeof(blindbutname) / sizeof(blindbutname[0]))

int BlindConsole::setbuttonon(button_name name) {
	int i = 0;

	cout << "\e[" << BUTTONY << ";1H";
	while (i < SIZE_BLINDBUTNAME) {
		if (blindbutname[i].but == name)
			cout << blindbutname[i].name << "\e[K";
		i++;
	}

	return 0;
}

int BlindConsole::help(int outdev) {
	int i;
	char *c;

	dprintf(outdev, "Help for blind terminal.\n");
	dprintf(outdev, "The following keys can be used:\n");

	i = 0;
	while (i < SIZE_BLIND_KEY) {
		if (blind_key[i].name) {
			c = &(blind_key[i].name[0]);
			while (islower(*c) && *c != '\0') {
				c++;
			}
			dprintf(outdev, "  %c %s\n", tolower(*c), blind_key[i].name);
		}
		i++;
	}

	return 0;
}

