/*
 * Speak Recorder
 * Copyright  2003 by Schou Industries ApS.
 */

#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#include "getoption.h"

GetOption::GetOption(int argc, char **argv, struct Get_long_options *new_long_options) {
	int option_index = 0;
	char *so = (char *)&short_options;
	char c = 0;
	int i;

	status = 0;
	glo = new_long_options;

	entries = 0;
	while (glo[entries].name != NULL || glo[entries].val != 0)
		entries++;
	entries++;

	long_options = (struct option *)malloc(entries*sizeof(option));
	short_options = (char *)malloc(entries*2+1);
	
	memset(long_options, 0, entries*sizeof(option));

	i = 0;
	so = short_options;
	while (i < entries) {
		memcpy(&long_options[i], &glo[i], sizeof(option));
		*so = long_options[i].val & 0x7f;
		if (*so)
			so++;
		if (long_options[i].has_arg)
			*so++ = ':';
		i++;
	}
	*so = 0;

	do {
		c = getopt_long(argc, argv, short_options, long_options, &option_index);
		i = 0;
		while (i < entries) {
			if (c == '?')
				status = 1;
			else if (c == glo[i].val) {
				if (glo[i].variable) {
					switch (glo[i].has_arg) {
						case 0:
							*(int *)glo[i].variable = 1;
							break;
						case 1:
							*(int *)glo[i].variable = atoi(optarg);
							break;
						case 2:
							*(char **)glo[i].variable = optarg;
							break;
					}
				}
			}
			i++;
		}
	} while (c != -1);
}

GetOption::~GetOption() {
	free(long_options);
	free(short_options);
}

void GetOption::help(int outdev) {
	int i = 0;
	char *opt_help;

	while (glo[i].name || glo[i].val) {
		opt_help = *(char **)glo[i].variable;
		dprintf(outdev, "  ");
		if (glo[i].val) {
			dprintf(outdev, "-%c", glo[i].val);
			switch (glo[i].has_arg) {
				case 1:
					if (opt_help) {
						dprintf(outdev, " %d", opt_help);
					} else {
						dprintf(outdev, " <int>");
					}
					break;
				case 2:
					dprintf(outdev, " %s", opt_help ? opt_help : "<string>");
					break;
				//dprintf(outdev, " %s", opt_help ? opt_help : (glo[i].has_arg==1 ? "<int>" : "<string>"));
			}
			if (strlen(glo[i].name)) {
				dprintf(outdev, ", ");
			}
		}
		if (strlen(glo[i].name)) {
			dprintf(outdev, "--%s", glo[i].name);
			switch (glo[i].has_arg) {
				case 1:
					if (opt_help) {
						dprintf(outdev, "=%d", opt_help);
					} else {
						dprintf(outdev, "=<int>");
					}
					break;
				case 2:
					dprintf(outdev, "=%s", opt_help ? opt_help : "<string>" );
					break;
				//dprintf(outdev, "=%s", opt_help ? opt_help : (glo[i].has_arg==1 ? "<int>" : "<string>") );
			}
		}
		if (glo[i].help)
			dprintf(outdev, "\n%s", glo[i].help);
			
		dprintf(outdev, "\n");
		i++;
	}
}

#ifndef HAVE_CONFIG_H
/* --------------------------------------------------------------- */

int help = 0;
int blind = 7;
int verbose = 0;
int version = 0;
char *mousedev = "/default/mouse";
char *joy_name = NULL;
char *tapebox = NULL;

struct Get_long_options help_long_options[] = {
	{"help",    0, 0, 'h', &help,     "This help." },
	{"blind",   1, 0, 'b', &blind,    "Enable blind mode. TTY output." },
	{"mouse",   2, 0, 'm', &mousedev, "Enable mouse input." },
	{"joy",     2, 0, 'j', &joy_name, "Enable joystick input." },
	{"",        2, 0, 't', &tapebox,  0 },
	{"",        0, 0, 'v', &verbose,  "Verbose." },
	{"version", 0, 0,   0, &version,  "Version of the program." },
	{0,0,0,0}
};

int main(int argc, char **argv ) {

	GetOption getoption(argc, argv, help_long_options);

	if (help) getoption.help(stdout);

	printf("\nblind: %d\n", blind);
	printf("version: %d\n", version);
	printf("Mouse device: %s\n", mousedev);

	return 0;
}
#endif

