/*
 * Speak Recorder
 * Copyright  2003 by Schou Industries ApS.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>

#include "speakrec.h"
#include "joystick.h"

#define NAMESIZE 100

/*
 * Logitech Inc. WingMan RumblePad
 *
 *   7             8
 * +-*-------------*-+
 * |           4 5 6 |
 * |   9       1 2 3 |
 * |      L   R      |
 * |  +-----------+  |
 * | /             \ |
 * |/               \|
 *
 */

struct {
	tape_key pres;
	tape_key rel;
	char *name;
} jbut[] = {
	{ key_none,        key_none, "Menu"      }, /* but 1 */
	{ key_play,        key_none, "Play"      }, /* but 2 */
	{ key_stop,        key_none, "Stop"      }, /* but 3 */
	{ key_rewind_inc,  key_none, "Rewind"    }, /* but 4 */
	{ key_forward_inc, key_none, "Forward"   }, /* but 5 */
	{ key_track_next,  key_none, "Tape"      }, /* but 6 */
	{ key_record,      key_none, "Record"    }, /* but 7 */
	{ key_edit,        key_stop, "Edit"      }, /* but 8 */
	{ key_quit,        key_none, "Quit"      }, /* but 9 */
	{ key_none,        key_none, "undef"     }  /* but 10 */
};

#define SIZE_JBUT (sizeof(jbut) / sizeof(jbut[0]))

#define CONSOLE_NAME "Joystick"

JoyConsole::JoyConsole(char *joyname) {
	nameofdevice = NULL;
	if (0 > (fd = open(joyname, O_RDONLY))) {
		cout << "joy " << fd << " open" << endl;
		exit(1);
		sleep(1);
	} else {
		char joynameioctl[NAMESIZE+1];
		if (ioctl(fd, JSIOCGNAME(NAMESIZE), joynameioctl)>=0) {
			nameofdevice = (char *)malloc(strlen(CONSOLE_NAME)+3+strlen(joyname)+3+strlen(joynameioctl)+1);
			strcat(nameofdevice, CONSOLE_NAME);
			strcat(nameofdevice, ", ");
			strcat(nameofdevice, joyname);
			strcat(nameofdevice, ", ");
			strcat(nameofdevice, (char *)joynameioctl);
		}
	}
}

JoyConsole::~JoyConsole(void) {
	free(nameofdevice);
	close(fd);
}

tape_key JoyConsole::getcommand(fd_set *read_set, tape_key lastcmd) {
	int i;
	tape_key key = lastcmd;

	if (FD_ISSET(fd, read_set)
			&& (read(fd, &js, sizeof(js)) == sizeof(js))) {
		if (js.type & JS_EVENT_INIT) {
			/* it is start-up time */
		}
		switch (js.type & (JS_EVENT_BUTTON | JS_EVENT_AXIS)) {
			case JS_EVENT_BUTTON:
				if (js.number < SIZE_JBUT) {
					if (js.value) {
						key = jbut[js.number].pres;
					} else {
						key = jbut[js.number].rel;
					}
				}
				break;
			case JS_EVENT_AXIS:
				break;
		}
		memcpy(&js_old, &js, sizeof(js_old));
	}

	return key;
}

int JoyConsole::fd_set_read(fd_set *read_set) {
	FD_SET(fd, read_set);

	return fd;
}

char wingman_layout[] =
"Logitech Inc. WingMan RumblePad\n"
"\n"
"     7           8\n"
"  +-###---------###-+\n"
"  |           4 5 6 |\n"
"  |   9       1 2 3 |\n"
"  |      L   R      |\n"
"  |   +---------+   |\n"
"  |  /           \\  |\n"
"  | /             \\ |\n"
"  |/               \\|\n"
"\n";

int JoyConsole::help(int outdev) {
	char joyname[NAMESIZE+1];
	char c_num_axis = 0;
	char c_num_buttons = 0;
	unsigned long version = 0;
	int i;

	dprintf(outdev, "Help for joystick\n");
	dprintf(outdev, "-----------------\n\n");

	if (fd) {
		if (ioctl(fd, JSIOCGNAME(NAMESIZE), joyname)>=0)
			dprintf(outdev, "Joystick model: \"%s\"\n", joyname);
		if (ioctl(fd, JSIOCGAXES, &c_num_axis))
			c_num_axis = 0;
		if (ioctl(fd, JSIOCGBUTTONS, &c_num_buttons))
			c_num_buttons = 0;
		if (!ioctl(fd, JSIOCGVERSION, &version))
			dprintf(outdev, "Joystick driver version: %d.%d.%d.\n",
				version >> 16, (version >> 8) & 0xff, version & 0xff);
		dprintf(outdev, "Number of buttons: %d\n", c_num_buttons);
		dprintf(outdev, "Number of axis: %d\n", c_num_axis);
	}
	dprintf(outdev, "\nButton layout:\n");
	dprintf(outdev, wingman_layout);
	i = 0;
	while (i < SIZE_JBUT) {
		if (jbut[i].pres != key_none) {
			dprintf(outdev, "  %d %s", i+1, jbut[i].name );
			if (jbut[i].rel == key_stop)
				dprintf(outdev, " and stop on release" );
			dprintf(outdev, "\n");
		}
		i++;
	}

	return 0;
}

