/*
 * Speak Recorder
 * Copyright  2003 by Schou Industries ApS.
 */

#include <iostream>
#include <unistd.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "speakrec.h"
#include "linkconsole.h"

using namespace std;

/*
 * linked list
 */

LinkConsole::LinkConsole() {
	higest_fd = 0;
	head = NULL;
	tail = NULL;
	nameofdevice = "Linked list. Not a console.";
}

LinkConsole::~LinkConsole() {
	CConsole *temp;

	while (temp = head) {
		head = head->next;
		delete temp;
	}
}

int LinkConsole::addconsole(CConsole *newcons) {
	if (newcons) {
		newcons->next = NULL;
		if (tail)
			tail->next = newcons;
		else
			head = newcons;
		tail = newcons;
	}

	return newcons == NULL;
}

int LinkConsole::showsegmenttext(const char *text) {
	char *old = (char *)currentsegmentbuf;
	int i = 0;

	while (i<NUM_SEGMENTS && *text != '\0') {
		if (*old != *text)
			showsegment(i, *old = *text);
		old++;
		text++;
		i++;
	}
	while (i<NUM_SEGMENTS) {
		if (*old != ' ') {
			*old = ' ';
			showsegment(i, *old);
		}
		old++;
		i++;
	}

	return strlen(text);
}

int LinkConsole::flush() {
	CConsole *temp = head;

	while (temp) {
		temp->flush();
		temp = temp->next;
	}
}

tape_key LinkConsole::getcommand(fd_set *read_set, tape_key lastcmd) {
	CConsole *temp = head;
	tape_key key = key_none;
	tape_key key_chk = key_none;

	while (temp) {
		if (key_none != (key_chk = temp->getcommand(read_set, key)))
			key = key_chk;
		temp = temp->next;
	}

	return key;
}

int LinkConsole::fd_set_read(fd_set *read_set) {
	CConsole *temp = head;
	int hfd;

	while (temp) {
		hfd = temp->fd_set_read(read_set);
		if (higest_fd < hfd)
			higest_fd = hfd;
		temp = temp->next;
	}

	return higest_fd;
}

int LinkConsole::fd_set_write(fd_set *write_set) {
	CConsole *temp = head;

	while (temp) {
		temp->fd_set_write(write_set);
		temp = temp->next;
	}

	return 0;
}

int LinkConsole::redraw(void) {
	CConsole *temp = head;

	while (temp) {
		temp->redraw();
		temp = temp->next;
	}

	return 0;
}

int LinkConsole::setvolume(int newvol, int peakvol) {
	CConsole *temp = head;

	while (temp) {
		temp->setvolume(newvol, peakvol);
		temp = temp->next;
	}

	return 0;
}

int LinkConsole::showsegment(int num, unsigned char value) {
	CConsole *temp = head;

	while (temp) {
		temp->showsegment(num, value);
		temp = temp->next;
	}

	return 0;
}

int LinkConsole::setbuttonon(button_name name) {
	CConsole *temp = head;

	while (temp) {
		temp->setbuttonon(name);
		temp = temp->next;
	}

	return 0;
}

char help_delimiter[] =
"---------------------------------------------------------------------------\n\n";

int LinkConsole::help(int outdev) {
	CConsole *temp = head;

	while (temp) {
		dprintf(outdev, help_delimiter);
		temp->help(outdev);
		dprintf(outdev, "\n");
		temp = temp->next;
	}
	dprintf(outdev, help_delimiter);

	return 0;
}
