/*
 * Speak Recorder
 * Copyright  2003 by Schou Industries ApS.
 */

/*
 * logvoume.cpp
 */

#include <stdlib.h>
#include "logvolume.h"

unsigned short rawlogtable[] = {
    0, /*  0 -30dB */
 1037, /*  1 -29dB */
 1163, /*  2 -29dB */
 1232, /*  3 -28dB */
 1305, /*  4 -28dB */
 1382, /*  5 -27dB */
 1464, /*  6 -27dB */
 1551, /*  7 -26dB */
 1643, /*  8 -26dB */
 1740, /*  9 -25dB */
 1843, /* 10 -25dB */
 1952, /* 11 -24dB */
 2068, /* 12 -24dB */
 2190, /* 13 -23dB */
 2320, /* 14 -23dB */
 2458, /* 15 -22dB */
 2603, /* 16 -22dB */
 2757, /* 17 -21dB */
 2921, /* 18 -21dB */
 3094, /* 19 -20dB */
 3277, /* 20 -20dB */
 3471, /* 21 -19dB */
 3677, /* 22 -19dB */
 3895, /* 23 -18dB */
 4126, /* 24 -18dB */
 4370, /* 25 -17dB */
 4629, /* 26 -17dB */
 4903, /* 27 -16dB */
 5194, /* 28 -16dB */
 5501, /* 29 -15dB */
 5827, /* 30 -15dB */
 6173, /* 31 -14dB */
 6538, /* 32 -14dB */
 6926, /* 33 -13dB */
 7336, /* 34 -13dB */
 7771, /* 35 -12dB */
 8231, /* 36 -12dB */
 8719, /* 37 -11dB */
 9235, /* 38 -11dB */
 9783, /* 39 -10dB */
10362, /* 40 -10dB */
10976, /* 41  -9dB */
11627, /* 42  -9dB */
12316, /* 43  -8dB */
13045, /* 44  -8dB */
13818, /* 45  -7dB */
14637, /* 46  -7dB */
15504, /* 47  -6dB */
16423, /* 48  -6dB */
17396, /* 49  -5dB */
18427, /* 50  -5dB */
19519, /* 51  -4dB */
20675, /* 52  -4dB */
21900, /* 53  -3dB */
23198, /* 54  -3dB */
24572, /* 55  -2dB */
26028, /* 56  -2dB */
27570, /* 57  -1dB */
29204, /* 58  -1dB */
30935, /* 59   0dB */
32767  /* 60   0db */
};

int CLogVolume::calcmap(void) {
	int i;
	int n = 0;

	wavtable = (char *)malloc(32768);
	for (i=0; i<32768; i++) {
		if (n+1<sizeof(rawlogtable) && rawlogtable[n+1] == i)
			n++;
		wavtable[i] = n;
	}

	return 0;
}

/**
 * logvol()
 * Input:
 * 	Volume in range 0..32767
 * Return:
 * 	LogVolume in range 0..60
 *
 * Formula:
 *	logvol = 60 + log10(decvol/32767.0)*20;
 */

int CLogVolume::logvol(int decvol) {

	if (decvol < 0)
		decvol = -decvol;
	if (decvol > 32767)
		decvol = 32767;
	
	return wavtable[decvol];
}

#ifndef HAVE_CONFIG_H
#include <stdio.h>
int main(void) {
	CLogVolume V;
	int i;
	int vol = 0;
	int dB = -60;
	int start = 0;
	int end = 0;

	for (i=0; i<32768; i++) {
		if (vol != V.logvol(i)) {
			dB++;
		}
		vol = V.logvol(i);
		printf("%5d %2d /* %3d */\n", i, vol, dB );
	}

	return 0;
}
#endif

