/*
 * Speak Recorder
 * Copyright  2003 by Schou Industries ApS.
 */

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/soundcard.h>
#include <fcntl.h>
#include "mixer.h"

CMixer::CMixer(char *mixerdevice, int *opt_pcm, int *opt_line, int *opt_igain) {
	int volpcm;

	fd = open(mixerdevice, O_RDWR);
	if (fd == -1) {
		devmask = 0;
		//write_log("Error open mixer: ", opt_devmixer);
	} else {
		pcm = opt_pcm;
		line = opt_line;
		igain = opt_igain;
		*pcm &= 0xFF;
		*pcm |= *pcm << 8;
		*line &= 0xFF;
		*line |= *line << 8;
		*igain &= 0xFF;
		*igain |= *igain << 8;
		//write_log("Mixer open: ", opt_devmixer);
		ioctl(fd, SOUND_MIXER_READ_DEVMASK, &devmask);
		if ((1 << SOUND_MIXER_PCM) & devmask) {
			ioctl(fd, MIXER_READ(SOUND_MIXER_PCM), &volpcm);
			//write_logi("Mixer PCM volume left: ", mixervolpcm & 0xFF);
		} else {
			volpcm = 0;
			//write_log("Warning, mixer no PCM volume", 0);
		}
	}
	set(stop);
}

CMixer::~CMixer(void) {
	set(mixclose);
}

int CMixer::set(mixer_mode setmode) {
	int vol = 0;

	switch (setmode) {
		case playback:
			if ((1 << SOUND_MIXER_PCM) & devmask) {
				ioctl(fd, MIXER_WRITE(SOUND_MIXER_PCM), pcm);
			}
			if ((1 << SOUND_MIXER_LINE) & devmask) {
				ioctl(fd, MIXER_WRITE(SOUND_MIXER_LINE), &vol);
			}
			if ((1 << SOUND_MIXER_IGAIN) & devmask) {
				ioctl(fd, MIXER_WRITE(SOUND_MIXER_IGAIN), &vol);
			}
			break;
		case stop:
		case record:
			if ((1 << SOUND_MIXER_PCM) & devmask) {
				ioctl(fd, MIXER_WRITE(SOUND_MIXER_PCM), &vol);
			}
			if ((1 << SOUND_MIXER_LINE) & devmask) {
				ioctl(fd, MIXER_WRITE(SOUND_MIXER_LINE), line);
			}
			if ((1 << SOUND_MIXER_IGAIN) & devmask) {
				ioctl(fd, MIXER_WRITE(SOUND_MIXER_IGAIN), igain);
			}
			break;
		case mixclose:
			if ((1 << SOUND_MIXER_PCM) & devmask) {
				ioctl(fd, MIXER_WRITE(SOUND_MIXER_PCM), pcm);
			}
			if ((1 << SOUND_MIXER_LINE) & devmask) {
				ioctl(fd, MIXER_WRITE(SOUND_MIXER_LINE), line);
			}
			if ((1 << SOUND_MIXER_IGAIN) & devmask) {
				ioctl(fd, MIXER_WRITE(SOUND_MIXER_IGAIN), igain);
			}
			break;
	}

	return 0;
}

