/*
 * Speak Recorder
 * Copyright  2003 by Schou Industries ApS.
 *
 * http://www.opensound.com/pguide/audio.html
 * http://www.josuttis.com/libbook/idx.html
 * http://www.4front-tech.com/pguide/audio.html
 */

//#define _FILE_OFFSET_BITS 64

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <fcntl.h>  /* O_RDWR */
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <dirent.h>
#include <sys/vfs.h>
#include <errno.h>

#include "speakrec.h"
#include "cconsole.h"
#include "linkconsole.h"
#include "termconsole.h"
#include "tapebox.h"
#include "blind.h"
#include "mouse.h"
#include "joystick.h"
#include "logvolume.h"
#include "getoption.h"
#include "menu.h"
#include "mixer.h"

#define STARTDIR "."
#define LOG_FILE_NAME PACKAGE ".log"

#define SOUND_SAMPLESIZE 16
#define SOUND_CHANNELS 1
#define SOUND_SPEED 44100

#if SOUND_SAMPLESIZE == 16
#  if SOUND_CHANNELS == 1
#    define BUF_TYPE short
#  endif
#endif

#define BYTES_PER_SEC (SOUND_SAMPLESIZE/8 * SOUND_CHANNELS * SOUND_SPEED)
#define BYTES_PER_SEC10 (BYTES_PER_SEC / 10)
#define BUFFER_SIZE (4096/1)
#define BUFFER_ITEMS BUFFER_SIZE/sizeof(BUF_TYPE)

#define PEAK_SAMPLES 20

#define NUM_BUFFERS 32

#define CUT_OUT_LATENCY (1+7)

#define REOPEN_SOUND_COUNT 20

#if _FILE_OFFSET_BITS == 64
#	define	MAX_REC_TIME "79.584.002 years" /* 2^63/(BYTES_PER_SEC*3600*365) */
#else
#	define	MAX_REC_TIME "6.75 hours" /* 2147483647L/(BYTES_PER_SEC*3600) */
#endif

#define max(a,b) (a > b ? a : b)

#define FILE_MODE_READ (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
#define FILE_MODE_WRITE (S_IRUSR | S_IWUSR)

#define MIX_PCM_ON 1
#define MIX_PCM_OFF 0

unsigned int beat = 0;
unsigned int beat2 = 0;
unsigned int reopen_sound_count = 0;

off_t counter_size = 0;
off_t counter_index = 0;

int opt_help = 0;
int opt_help_term = 0;
int opt_blind = 0;
int opt_verbose = 0;
int opt_version = 0;
int opt_spoolback = 40960/sizeof(BUF_TYPE);
int opt_revspeed = 5;
int opt_cutin = 8000;
int opt_cutplay = 50600;
int opt_lf30hz = 1;
int opt_pcm = 90;
int opt_line = 90;
int opt_igain = 90;
char *opt_devmouse = NULL;
char *opt_devjoy = NULL;
char *opt_tapebox1 = NULL;
char *opt_tapebox2 = NULL;
char *opt_devdsp = "/dev/dsp";
char *opt_devmixer = "/dev/sound/mixer";
char *opt_command = NULL;
char *opt_debug = NULL;

struct Get_long_options help_long_options[] = {
	{"help",     0, 0, 'h', &opt_help,     "\tThis help." },
	{"help-cons",0, 0, 'C', &opt_help_term,"\tDetailed help about console devices.\n\tOnly help for loaded devices will be shown.\n\tTry '-C -m /dev/input/mice' for help on mouse." },
	{"command",  2, 0, 'c', &opt_command,  "\tCommand at startup. Default is 'stop'. --command={stop,record}" },
	{"spoolback",1, 0, 's', &opt_spoolback,"\tSpool samples back in current recording on stop and record.\n\tTry with 20000 (about 0.5 second)." },
	{"cutin",    1, 0, 'i', &opt_cutin,    "\tNumber of samples in sound dev buffer to skip when start recording from stop." },
	{"cutplay",  1, 0, 'p', &opt_cutplay,  "\tNumber of samples in sound dev buffer to skip when start recording from play." },
	{"revspeed", 1, 0, 'r', &opt_revspeed, "\tReverse speed when 'Edit'." },
	{"dspaudio", 2, 0, 'd', &opt_devdsp,   "\tChange audio device." },
	{"devmixer", 2, 0, 'M', &opt_devmixer, "\tAudio mixer." },
	{"pcm",      1, 0, 'P', &opt_pcm,      "\tMixer PCM level during play-back." },
	{"line",     1, 0, 'L', &opt_line,     "\tMixer Line level during stop/record." },
	{"igain",    1, 0, 'I', &opt_igain,    "\tMixer Igain level during stop/record." },
	{"tapebox1", 2, 0, 't', &opt_tapebox1, "\tEnable external tapebox1. I.e. /dev/tts/USB0 /dev/usb/tts/0 or /dev/ttyUSB0\n\tfor first USB serial port." },
	{"tapebox2", 2, 0, 'T', &opt_tapebox2, "\tEnable external tapebox2. I.e. /dev/usb/tts/1 for second USB serial port." },
	{"mouse",    2, 0, 'm', &opt_devmouse, "\tEnable mouse input. --mouse=/dev/input/mice" },
	{"joy",      2, 0, 'j', &opt_devjoy,   "\tEnable joystick input. --joy=/dev/input/js0" },
	{"blind",    0, 0, 'b', &opt_blind,
#ifdef BLIND
		"\tDisable blind mode and use full screen color mode."
#else
		"\tEnable blind mode. TTY output."
#endif
	},
	{"lf30hz",  0, 0, 'l', &opt_lf30hz,   "\tLow frequency 30Hz added in the end of each file." },
	{"debug",   1, 0, 'D', &opt_debug,    "\tEnable debug mode. Default '0'.\n\t1: Flash\n\t2: Run light" },
	{"verbose", 0, 0, 'v', &opt_verbose,  "\tVerbose. Combine with 'h' to get detailed help about console." },
	{"version", 0, 0, 'V', &opt_version,  "\tProgram version." },
	{0,0,0,0}
};

struct {
	tape_key key;
	tape_action action;
	button_name button;
} actionbutton[] = {
	{ key:key_quit,        action:tape_quit,        button:but_none       },
	{ key:key_stop,        action:tape_stop,        button:but_stop       },
	{ key:key_record,      action:tape_record,      button:but_record     },
	{ key:key_play,        action:tape_play,        button:but_play       },
	{ key:key_edit,        action:tape_edit,        button:but_edit       },
	{ key:key_rewind_inc,  action:tape_rewind_1,    button:but_rewind     },
	{ key:key_forward_inc, action:tape_forward_1,   button:but_forward    },
	{ key:key_track_next,  action:tape_track_next,  button:but_track_next },
	{ key:key_track_prev,  action:tape_track_prev,  button:but_track_prev },
	{ key:key_menu,        action:tape_menu,        button:but_menu       }
};

#define SIZE_ACTIONBUTTON (sizeof(actionbutton) / sizeof(actionbutton[0]))

struct {
	tape_action action;
	char *name;
} actionnames[] = {
	{ tape_noaction,   "noaction"   },
	{ tape_stop,       "stop"       },
	{ tape_play,       "play"       },
	{ tape_record,     "record"     },
	{ tape_track_prev, "track_prev" },
	{ tape_track_next, "track_next" },
	{ tape_rewind_1,   "rewind_1"   },
	{ tape_rewind_2,   "rewind_2"   },
	{ tape_rewind_3,   "rewind_3"   },
	{ tape_rewind_4,   "rewind_4"   },
	{ tape_rewind_6,   "rewind_6"   },
	{ tape_rewind_12,  "rewind_12"  },
	{ tape_forward_1,  "forward_1"  },
	{ tape_forward_2,  "forward_2"  },
	{ tape_forward_3,  "forward_3"  },
	{ tape_forward_4,  "forward_4"  },
	{ tape_forward_6,  "forward_6"  },
	{ tape_forward_12, "forward_12" },
	{ tape_edit,       "edit"       },
	{ tape_quit,       "quit"       }
};

#define SIZE_ACTIONNAMES sizeof(actionnames) / sizeof(actionnames[0])

char *actionname(tape_action action) {
	int i = 0;

	while (i < SIZE_ACTIONNAMES && actionnames[i].action != action)
		i++;
	if (i < SIZE_ACTIONNAMES)
		return actionnames[i].name;

	return actionnames[0].name;
}

struct {
	unsigned int label;
	char *name;
} afmt[] = {
	{ AFMT_MU_LAW,	"AFMT_MU_LAW"	},
	{ AFMT_A_LAW,	"AFMT_A_LAW"	},
	{ AFMT_IMA_ADPCM,"AFMT_IMA_ADPCM"},	
	{ AFMT_U8,	"AFMT_U8"	},	
	{ AFMT_S16_LE,	"AFMT_S16_LE"	},
	{ AFMT_S16_BE,	"AFMT_S16_BE"	},
	{ AFMT_S8,	"AFMT_S8"	},	
	{ AFMT_U16_LE,	"AFMT_U16_LE"	},
	{ AFMT_U16_BE,	"AFMT_U16_BE"	},
	{ AFMT_MPEG,	"AFMT_MPEG"	},
	{ AFMT_AC3,	"AFMT_AC3"	}
};

#define SIZE_AFMT sizeof(afmt) / sizeof(afmt[0])

/* SNDCTL_DSP_STEREO:
 *   http://www.opensound.com/pguide/audio.html
 *   It is important to notice that only values 0 and 1 are allowed.
 *   Result of using any other value is undefined. 
 */

int bufused(int head, int tail) {
	return (NUM_BUFFERS+head - tail) % NUM_BUFFERS;
}

int buffree(int head, int tail) {
	return (NUM_BUFFERS+tail - head - 1) % NUM_BUFFERS;
}

struct {
	int set;
	int request;
	int value;
	int org;
	char *name;
} dsp_ioctl[] = {
	/*
	{ request:SNDCTL_DSP_RESET,      value:0,                org:0,                name:"SNDCTL_DSP_RESET" },
	*/
	{ set:1, request:SNDCTL_DSP_SYNC,       value:0,                org:0,                name:"SNDCTL_DSP_SYNC" },
	{ set:1, request:SNDCTL_DSP_CHANNELS,   value:SOUND_CHANNELS,   org:SOUND_CHANNELS,   name:"SNDCTL_DSP_CHANNELS" },
	{ set:1, request:SNDCTL_DSP_SAMPLESIZE, value:SOUND_SAMPLESIZE, org:SOUND_SAMPLESIZE, name:"SNDCTL_DSP_SAMPLESIZE" },
	{ set:1, request:SNDCTL_DSP_SPEED,      value:SOUND_SPEED,      org:SOUND_SPEED,      name:"SNDCTL_DSP_SPEED" },
	{ set:0, request:SNDCTL_DSP_GETBLKSIZE, value:0,                org:0,                name:"SNDCTL_DSP_GETBLKSIZE" },
	{ set:1, request:SNDCTL_DSP_SYNC,       value:0,                org:0,                name:"SNDCTL_DSP_SYNC" }
};

#define SIZE_DSP_IOCTL (sizeof(dsp_ioctl) / sizeof(dsp_ioctl[0]))

typedef enum {
	mode_stop,
	mode_record,
	mode_play,
	mode_rewind,
	mode_forward
} tape_mode;

typedef enum {
	nextname,
	prevname,
	newname,
} filenamedirection;

struct riff_hdr {
	char riff[4]; /* identifier string = "RIFF" */
	unsigned long remLen; /* remaining length after this header */
	char wave[4]; /* identifier string = "WAVE" */
	char fmt_[4]; /* identifier string = "fmt" */
	long           chunkSize;
	short          wFormatTag;
	unsigned short wChannels;
	unsigned long  dwSamplesPerSec;
	unsigned long  dwAvgBytesPerSec;
	unsigned short wBlockAlign;
	unsigned short wBitsPerSample;
	char id4[4]; /* data */
	long dataSize;
};

struct riff_hdr wavheader = {
	'R','I','F','F',
	2*4 + 4 + 16 + 2*4 /* + dataSize */ , /* remLen */
	'W','A','V','E',
	'f','m','t',' ',
	16, /* chunkSize */
	1, /* wFormatTag */
	SOUND_CHANNELS, /* wChannels */
	SOUND_SPEED, /* dwSamplesPerSec */
	SOUND_SPEED*SOUND_CHANNELS*SOUND_SAMPLESIZE/8, /* dwAvgBytesPerSec */
	SOUND_CHANNELS*SOUND_SAMPLESIZE/8, /* wBlockAlign */
	SOUND_SAMPLESIZE, /* wBitsPerSample */
	'd','a','t','a', /* id4 */
	0 /* dataSize */
};

struct bufset {
	int count;
	BUF_TYPE data[BUFFER_ITEMS];
};

class CSoundIO {
public:
	LinkConsole *terminal;
	int fd_file;
	int fd_dsp;
	int fd_log;
	struct bufset buffers[NUM_BUFFERS];
	BUF_TYPE temp_buf[BUFFER_ITEMS];
	unsigned short buf_head;
	unsigned short buf_tail;
	int end_of_file;
	int did_record;
	off_t current_file_pos;
	tape_mode mode;
	tape_mode lastmode;
	struct timeval timemark;
	BUF_TYPE maxvolume;
	BUF_TYPE peakvolume;
	BUF_TYPE peaksamples[PEAK_SAMPLES];
	unsigned int peakcount;
	int make_ramp_attack;
	int cut_in_latency;
	int cut_out;
	int playspeed;
	int read_dsp_stereo;
	char filename[20];
	char diskfree[NUM_SEGMENTS+1];
	char diskfreetime[NUM_SEGMENTS+1];
	long lastsecond;
	CMixer *mixer;

	CSoundIO(LinkConsole *newterminal);
	~CSoundIO(void);
	char *diskfreetext(void);
	int validwavheader(riff_hdr *hdr);
	int findfilename(filenamedirection direc);
	int valid_filename(char *name);
	off_t sumfilesize(int curr);
	int openfilename(char *name);
	int opennewfilename(char *name);
	int openexistingfilename(char *name);
	int openprevfilename(int fd);
	int openfiles(char *dspname);
	int closewavfile(void);
	int closefiles(void);
	int conv_stereo_mono(BUF_TYPE *buf, int num);
	int read_dsp(void);
	int write_dsp(void);
	int write_file(void);
	int read_file(void);
	int findmaxvolume(BUF_TYPE *buf, int buffer_count);
	int ramp_attack(int buf_num);
	int write_log(char *text, char *extra);
	int write_logi(char *text, long int value);
	BUF_TYPE peakadd(BUF_TYPE volume);
	int stoprecording(int doit);
	tape_action doaction(tape_action action);
	tape_mode domode(fd_set *rd_set, fd_set *wr_set);
	int fd_set_rw(fd_set *rd_set, fd_set *wr_set);
	long sec10(void);
	int playrespeed(struct bufset *bufsrc);
	int correctplayspeed(int ps);
	int displaysecond10(long sec10);
	tape_action getaction(fd_set *rd_set);
	tape_action takeaction(fd_set *rd_set, fd_set *wr_set);
	void reportterms(CConsole *term);
	void run_menu(void);
	int dispset(int col, fd_set *rd_set, fd_set *wr_set);
};

CSoundIO::CSoundIO(LinkConsole *newterminal) {

	terminal = newterminal;
	fd_file = 0;
	fd_dsp = 0;
	mode = mode_stop;
	gettimeofday(&timemark, NULL);
	bzero(&peaksamples, sizeof(peaksamples));
	maxvolume = 0;
	peakvolume = 0;
	buf_head = buf_tail = 0;
	make_ramp_attack = 0;
	cut_in_latency = 0;
	cut_out = 0;
	playspeed = 0;
	strncpy(filename, "9999.wav", sizeof(filename));
	counter_index = 0;
	lastsecond = -1;

	mixer = new CMixer(opt_devmixer, &opt_pcm, &opt_line, &opt_igain);
}

CSoundIO::~CSoundIO(void) {

	closefiles();
}

char * CSoundIO::diskfreetext(void) {
	char *bytetext[] = {
		"B", "KB", "MB", "GB", "TB", "PB", "EB"
	};
	int i = 0;
	struct statfs stat;
	unsigned long long available = 0;
	unsigned long long time = 0;

	i = statfs(STARTDIR, &stat);
	if (i) {
		diskfree[0] = '\0';
		diskfreetime[0] = '\0';
	} else {
		available = (unsigned long long)stat.f_bsize * (unsigned long long)stat.f_bavail;
		time = available / (BYTES_PER_SEC * 60LL);
		i = 0;
		while (available > 999) {
			available /= 1024;
			i++;
		}
		snprintf((char *)&diskfree, sizeof(diskfree), "DF%3llu%s", available, bytetext[i]);
		snprintf((char *)&diskfreetime, sizeof(diskfreetime),
				time < 9999 ? "Mi %4llu" : "Hr %4llu",
				time < 9999 ? time : (time / 60 < 999 ? time / 60 : 999)
				);
	}

	return (char *)&diskfree;
}

struct {
	int code;
	char *name;
} wavfileerror[] = {
	{ 1 <<  0, "Text 'RIFF' not found in file." },
	{ 1 <<  1, "Text 'WAVE' not found in file." },
	{ 1 <<  2, "Text 'fmt_' not found in file." },
	{ 1 <<  3, "Chunksize error." },
	{ 1 <<  4, "FormatTag error." },
	{ 1 <<  5, "wChannels error, only one channel (mono) is supported." },
	{ 1 <<  6, "dwSamplesPerSec error. Only 44100 samples per second is supported." },
	{ 1 <<  7, "dwAvgBytesPerSec error. Only 88200 bytes per second is supported." },
	{ 1 <<  8, "wBlockAlign error. Only 2 byte block align is supported." },
	{ 1 <<  9, "wBitsPerSample error. Only 16 bit per sample is supported." },
	{ 1 << 10, "ID error. Text 'data' not found in file." },
};

#define SIZE_WAVFILEERROR sizeof(wavfileerror) / sizeof(wavfileerror[0])

int CSoundIO::validwavheader(riff_hdr *hdr) {
	int errors = 0;

	errors |= memcmp(hdr->riff, wavheader.riff, sizeof(hdr->riff))
		? wavfileerror[ 0].code : 0;
	errors |= memcmp(hdr->wave, wavheader.wave, sizeof(hdr->wave))
		? wavfileerror[ 1].code : 0;
	errors |= memcmp(hdr->fmt_, wavheader.fmt_, sizeof(hdr->fmt_))
		? wavfileerror[ 2].code : 0;
	errors |= hdr->chunkSize != wavheader.chunkSize
		? wavfileerror[ 3].code : 0;
	errors |= hdr->wFormatTag != wavheader.wFormatTag
		? wavfileerror[ 4].code : 0;
	errors |= hdr->wChannels != wavheader.wChannels
		? wavfileerror[ 5].code : 0;
	errors |= hdr->dwSamplesPerSec != wavheader.dwSamplesPerSec
		? wavfileerror[ 6].code : 0;
	errors |= hdr->dwAvgBytesPerSec != wavheader.dwAvgBytesPerSec
		? wavfileerror[ 7].code : 0;
	errors |= hdr->wBlockAlign != wavheader.wBlockAlign
		? wavfileerror[ 8].code : 0;
	errors |= hdr->wBitsPerSample != wavheader.wBitsPerSample
		? wavfileerror[ 9].code : 0;
	errors |= memcmp(hdr->id4, wavheader.id4, sizeof(hdr->id4))
		? wavfileerror[10].code : 0;

	return errors;
}

off_t getsoundsize( char *filename ) {
	struct stat status;
	off_t size = 0;

	if (lstat(filename, &status) == 0) {
		size = status.st_size - sizeof(wavheader);
	}

	return size;
}

int CSoundIO::valid_filename(char *name) {
#define FEXT ".wav"
#define NFILENAMEDIGIT 4
#define SFILENAMEDIGIT "4"
	int i;

	if (strlen(name) != NFILENAMEDIGIT+strlen(FEXT)) {
		return 0;
	}
	if (strncmp(name+NFILENAMEDIGIT, FEXT, strlen(FEXT))) {
		return 0;
	}
	i = NFILENAMEDIGIT;
	while (i--) {
		if (!isdigit(name[i])) {
			return 0;
		}
	}

	return 1;
}

off_t CSoundIO::sumfilesize(int curr) {
	int tmpval;
	struct dirent *dp;
	off_t sumsize = 0;

	if (curr > 1) {
		DIR *DP = opendir(STARTDIR);
		while (dp = readdir(DP)) {
			if (valid_filename(dp->d_name)) {
				tmpval = atoi(dp->d_name);
				if (tmpval < curr) {
					sumsize += getsoundsize(dp->d_name);
				}
			}
		}
		closedir(DP);
	}

	return sumsize;
}

int CSoundIO::findfilename(filenamedirection direc) {
	int curr = atoi(filename);
	int tmpval = 1;
	int prev = 1;
	int next = 10000;
	int high = 0;
	struct dirent *dp;

	DIR *DP = opendir(STARTDIR);
	while (dp = readdir(DP)) {
		if (valid_filename(dp->d_name)) {
			tmpval = atoi(dp->d_name);
			if (tmpval > curr) {
				if (tmpval < next) {
					next = tmpval;
				}
			} else {
				if (tmpval < curr) {
					if (tmpval > prev) {
						prev = tmpval;
					}
				}
			}
			if (tmpval > high) {
				high = tmpval;
			}
		}
	}
	closedir(DP);

	switch (direc) {
		case newname:
			tmpval = high + 1;
			break;
		case nextname:
			if (next > 9999) {
				tmpval = high + 1;
			} else {
				tmpval = next;
			}
			break;
		case prevname:
			tmpval = prev;
			break;
	}

	return tmpval;
}

int CSoundIO::openfilename(char *name) {
	int fd;

	write_log("Open file: ", name);
	chmod(name, FILE_MODE_WRITE);
	fd = open(name, O_RDWR | O_CREAT, FILE_MODE_WRITE);
	if (fd < 0) {
		fprintf(stderr, "Error, file open fd: %d '%s' at line &d\n", fd, name, __LINE__);
		exit(1);
	}
	lastsecond = -1;

	return fd;
}

int CSoundIO::opennewfilename(char *name) {
	int oldnumber = atoi(name);
	int val;
	int fd;

	val = findfilename(nextname);
	if (val == oldnumber || oldnumber == 0) {
		val = findfilename(newname);
	}

	closewavfile();

	snprintf(filename, sizeof(filename), "%0" SFILENAMEDIGIT "d" FEXT, val );
	fd = openfilename(filename);
	write(fd, &wavheader, sizeof(wavheader));

	counter_size = sumfilesize(atoi(filename));
	end_of_file = 0;
	current_file_pos = 0;
	did_record = 0;

	return fd;
}

int CSoundIO::openexistingfilename(char *name) {
	struct stat buf;
	int fd;

	if (0 == lstat(name, &buf)) {
		fd = openfilename(name);
	} else {
		fd = opennewfilename(name);
	}

	return fd;
}

int CSoundIO::openprevfilename(int fd) {
	int i;
	int curr;
	int prev;
	struct riff_hdr hdr;

	curr = atoi(filename);
	prev = findfilename(prevname);
	if (curr == prev) {
		current_file_pos = 0;
		write_log("Trying to go prev file: ", filename);
		return fd;
	}

	closewavfile();

	snprintf(filename, sizeof(filename), "%0" SFILENAMEDIGIT "d" FEXT, prev );
	fd = openfilename(filename);
	if ((curr = read(fd, &hdr, sizeof(hdr))) != sizeof(hdr)) {
		write_log("Error: can not open file for read, wrong size of header: ", filename);
		printf("\e[0m\nwrong size of header in %s, size: %d, should be: %d\n", filename, curr, sizeof(hdr));
		write(fd, &wavheader, sizeof(wavheader));
	} else {
		curr = validwavheader( &hdr );
		if (curr != 0) {
			write_log("Error in wavfile when read. Wrong header in file: ", filename);
			i = 0;
			while (i<SIZE_WAVFILEERROR) {
				if (curr & wavfileerror[i].code) {
					write_log("Wav file error: ", wavfileerror[i].name);
				}
				i++;
			}
			printf("\e[0m\nwrong header in %s, errors: %08x\n", filename, curr);
		}
	}

	counter_size = sumfilesize(atoi(filename));
	end_of_file = 0;
	current_file_pos = 0;
	current_file_pos = lseek(fd, 0, SEEK_END)-sizeof(hdr);

	return fd;
}

int CSoundIO::openfiles(char *dspname) {
	int i;
	char tmpbuf[200];

	fd_log = open(LOG_FILE_NAME, O_WRONLY | O_CREAT | O_APPEND, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
	if (fd_log < 0) {
		fprintf(stderr, "Error, file open fd_log: %d at line %d\n", fd_log, __LINE__);
		exit(1);
	}
	write_log("Log started for: ", PACKAGE" "VERSION );
	write_log("Disk free: ", diskfreetext() );
	write_log("Time free: ", diskfreetime );

	fd_dsp = open(dspname, O_RDWR);
	if (fd_dsp < 0) {
		snprintf((char *)tmpbuf, sizeof(tmpbuf)-1, "%s: %s", dspname, strerror(errno));
		write_log("Error, dsp open: ", tmpbuf);
		fprintf(stderr, "Error, file open %s: %d at line %d\n", dspname, fd_dsp, __LINE__);
		exit(1);
	}
	write_log("Audio device open: ", dspname);
	int dsp_delay;
	if (0>ioctl(fd_dsp, SNDCTL_DSP_GETODELAY, &dsp_delay)) {
		write_log("ioctl SNDCTL_DSP_GETODELAY: ", strerror(errno));
	} else {
		write_logi("SNDCTL_DSP_GETODELAY: ", dsp_delay);
	}
	i = 0;
	while (i<SIZE_DSP_IOCTL) {
		snprintf((char *)tmpbuf, sizeof(tmpbuf)-1, "%s request:%x, value:%d, org-value:%d",
					dsp_ioctl[i].name, dsp_ioctl[i].request, dsp_ioctl[i].value, dsp_ioctl[i].org);
		write_log("dsp ioctl: ", tmpbuf);
		if (ioctl(fd_dsp, dsp_ioctl[i].request, &dsp_ioctl[i].value) < 0) {
			write_log("Error dsp ioctl-1: ", tmpbuf);
			fprintf(stderr, "\rioctl error: %s request:%x, value:%d\n",
					dsp_ioctl[i].name, dsp_ioctl[i].request, dsp_ioctl[i].value);
			exit(1);
		}
		write_logi("dsp ioctl value: ", dsp_ioctl[i].value);
		if (dsp_ioctl[i].request == SNDCTL_DSP_CHANNELS) {
			write_log("setting SNDCTL_DSP_CHANNELS", NULL);
			switch (dsp_ioctl[i].value) {
				case 1:
					read_dsp_stereo = 0;
					write_log("Reading mono from dsp", NULL);
					break;
				case 2:
					read_dsp_stereo = 1;
					write_log("Reading stereo from dsp", NULL);
					break;
				default:
					exit(1);
			}
		}
		if (dsp_ioctl[i].set && dsp_ioctl[i].value != dsp_ioctl[i].org) {
			snprintf((char *)tmpbuf, sizeof(tmpbuf)-1,
					"%s, current value:%d, request value:%d\n",
					dsp_ioctl[i].name, dsp_ioctl[i].value, dsp_ioctl[i].org);
			write_log("Error dsp ioctl-2: ", tmpbuf);
			fprintf(stderr, "\rioctl error: %s, value:%d, org:%d\n",
				dsp_ioctl[i].name, dsp_ioctl[i].value, dsp_ioctl[i].org);
			//exit(1);
		}
		i++;
	}

	struct audio_buf_info info;
	memset(&info, 0, sizeof(info));
	if (0>ioctl(fd_dsp, SNDCTL_DSP_GETOSPACE, &info)) {
		write_log("ioctl SNDCTL_DSP_GETOSPACE: ", strerror(errno));
	} else {
		write_logi("SNDCTL_DSP_GETOSPACE.fragments: ", info.fragments);
		write_logi("SNDCTL_DSP_GETOSPACE.fragstotal: ", info.fragstotal);
		write_logi("SNDCTL_DSP_GETOSPACE.fragsize: ", info.fragsize);
		write_logi("SNDCTL_DSP_GETOSPACE.bytes: ", info.bytes);
	}
	memset(&info, 0, sizeof(info));
	if (0>ioctl(fd_dsp, SNDCTL_DSP_GETISPACE, &info)) {
		write_log("ioctl SNDCTL_DSP_GETISPACE: ", strerror(errno));
	} else {
		write_logi("SNDCTL_DSP_GETISPACE.fragments: ", info.fragments);
		write_logi("SNDCTL_DSP_GETISPACE.fragstotal: ", info.fragstotal);
		write_logi("SNDCTL_DSP_GETISPACE.fragsize: ", info.fragsize);
		write_logi("SNDCTL_DSP_GETISPACE.bytes: ", info.bytes);
	}

	int mask;
	int setmask = AFMT_S16_LE;
	if (ioctl(fd_dsp, SNDCTL_DSP_GETFMTS, &mask) != -1)
		for (i=0; i<SIZE_AFMT; i++)
			if (mask & afmt[i].label) {
				write_log("SNDCTL_DSP_GETFMTS: ", afmt[i].name);
				if (afmt[i].label == AFMT_S16_LE) {
					ioctl(fd_dsp, SNDCTL_DSP_SETFMT, &setmask);
					write_log("Setting: ", afmt[i].name);
				}
			}

	fd_file = opennewfilename(filename);

	return 0;
}

int CSoundIO::closewavfile(void) {

	if (fd_file > 0) {
		if (sizeof(wavheader) == lseek(fd_file, 0, SEEK_END)) {
			write_log("No data in wav-file. Deleting file: ", filename);
			close(fd_file);
			unlink(filename);
		} else {
			if (mode == mode_record) {
				while (buf_head != buf_tail) {
					write_file();
				}
			}
			off_t filesize = lseek(fd_file, 0, SEEK_END);
			lseek(fd_file, 0, SEEK_SET);
			wavheader.remLen = filesize-sizeof(wavheader.riff)-sizeof(wavheader.remLen);
			wavheader.dataSize = filesize-sizeof(wavheader);
			write(fd_file, &wavheader, sizeof(wavheader) );
			close(fd_file);
			chmod(filename, FILE_MODE_READ);
			char fs[100];
			snprintf((char *)&fs, sizeof(fs), "(%04dMB) %10d", filesize/(1024*1024), filesize);
			write_log("Wav file size: ", fs);
			write_log("Wav file closed: ", filename);
			if (did_record && opt_lf30hz) {
				did_record = 0;
				terminal->showsegmenttext("LF30 Hz");
				char *sysstr = (char *)malloc(2048);
				snprintf(sysstr, 2048-1, "nice -n 19 %s.lf30hz -q %s &",
						PACKAGE, filename);
				errno = 0;
				write_log("Adding Low Frequency 30Hz: ", sysstr);
				int sysres = system(sysstr);
				write_logi("Result from running 'system': ", sysres);
				if (errno) {
					fprintf(stderr, "Error, lf30hz errno: '%s'\n", strerror(errno));
				}
				free(sysstr);
			}
		}
		fd_file = 0;
	}

	return fd_file;
}

int CSoundIO::closefiles(void) {

	if (fd_dsp > 0) {
		close(fd_dsp);
		fd_dsp = 0;
	}

	closewavfile();

	if (fd_log > 0) {
		close(fd_log);
		fd_log = 0;
	}

	return 0;
}


int CSoundIO::playrespeed(struct bufset *bufsrc) {
	struct bufset tbuf;
	int frames; /* Number of samples */
	int count;

	if (playspeed == 0 || playspeed == 1) {
		return bufsrc->count;
	}

	BUF_TYPE *dst = &tbuf.data[0];
	BUF_TYPE *src = &bufsrc->data[ playspeed > 0 ? 0 : bufsrc->count/sizeof(BUF_TYPE)-1 ];

	count = bufsrc->count;
	bufsrc->count /= abs(playspeed);
	if (bufsrc->count % sizeof(BUF_TYPE)) {
		if (opt_verbose) {
			write_logi("ERROR: playrespeed odd count: ", bufsrc->count);
			write_logi("ERROR: playrespeed count was: ", count);
		}
		/* Make mono 16 bit (2 bytes per frame) */
		bufsrc->count &= ~(sizeof(BUF_TYPE)-1);
	}
	frames = bufsrc->count / sizeof(BUF_TYPE);
	
	while (frames--) {
		*dst++ = *src;
		src += playspeed;
	}
	
	memcpy(&bufsrc->data[0], &tbuf.data[0], bufsrc->count);

	return bufsrc->count;
}

int CSoundIO::conv_stereo_mono(BUF_TYPE *buf, int num) {
	BUF_TYPE *p1 = buf;
	BUF_TYPE *p2 = buf;
	int count = num / (SOUND_SAMPLESIZE/8 * 2);

	while (count--) {
		*p1 = *p2;
		p1++;
		p2++; p2++;
	}

	return num/2;
}

int CSoundIO::read_dsp(void) {
	int num_read;

	num_read = read(fd_dsp, &buffers[buf_head % NUM_BUFFERS].data, BUFFER_SIZE);
	if (read_dsp_stereo) {
		num_read = conv_stereo_mono((BUF_TYPE *)&buffers[buf_head % NUM_BUFFERS].data, num_read);
	}
	buffers[buf_head % NUM_BUFFERS].count = num_read;
	if (num_read > 0) {
		maxvolume = findmaxvolume(&buffers[buf_head % NUM_BUFFERS].data[0], num_read);
		buf_head++;
	} else {
		write_log("read(dsp) num_read==0 ?", 0);
		fprintf(stderr, "Error, num_read: '%s'\n", strerror(errno));
	}

	return num_read;
}

int CSoundIO::write_dsp(void) {
	int num_written = 0;

	/* if buffer dirty */
	if (buf_tail != buf_head) {
		num_written = write(fd_dsp, &buffers[buf_tail % NUM_BUFFERS].data, buffers[buf_tail % NUM_BUFFERS].count );
		if (num_written < 0) {
			write_log("ERROR, write_dsp: ", strerror(errno));
			write_logi("ERROR, write_dsp, buf_tail ", buf_tail);
			write_logi("ERROR, write_dsp, buf_head ", buf_head);
			write_logi("ERROR, write_dsp, buf_tail % NUM_BUFFERS ", buf_tail % NUM_BUFFERS);
			write_logi("ERROR, write_dsp, count ", buffers[buf_tail % NUM_BUFFERS].count);
		}
		maxvolume = findmaxvolume(&buffers[buf_tail % NUM_BUFFERS].data[0], num_written);
		buf_tail++;
		current_file_pos += num_written * playspeed;
		if (opt_verbose) {
			write_logi("Write: current_file_pos: ", current_file_pos);
		}
		if (current_file_pos < 0) {
			current_file_pos = 0;
			mode = mode_stop;
			write_log("ERROR: write_dsp negative file pos.", NULL);
		}
		if (buf_tail == buf_head) {
			write_log("write_dsp, no more sound in buffers, start reopen count down", NULL);
			reopen_sound_count = REOPEN_SOUND_COUNT;
		} else {
			reopen_sound_count = 0;
		}
	} else {
		mode = mode_stop;
		if (opt_verbose) {
			write_log("write_dsp: mode = mode_stop", 0);
		}
	}

	return num_written;
}

int CSoundIO::read_file(void) {
	int *num_read;
	long file_pos = -2;

	if (!end_of_file && (playspeed < 0) ) {
		file_pos = lseek(fd_file, -BUFFER_SIZE, SEEK_CUR);
		if (file_pos <= sizeof(wavheader)) {
			end_of_file = 1;
			file_pos = lseek(fd_file, sizeof(wavheader), SEEK_SET);
		}
	}
	if (!end_of_file) {
		/* if any free buffers */
		if ( (NUM_BUFFERS+buf_tail-buf_head-1) % NUM_BUFFERS ) {
			num_read = &buffers[buf_head % NUM_BUFFERS].count;
			*num_read = read(fd_file, &buffers[buf_head % NUM_BUFFERS].data, BUFFER_SIZE);
			if (opt_verbose && *num_read % 2) {
				write_logi("ERROR, read odd from file, num_read: ", *num_read);
				/* Make even number of bytes */
				*num_read &= ~(sizeof(BUF_TYPE)-1);
				write_logi("ERROR, num_read corrected: ", *num_read);
			}
			if (opt_verbose && *num_read != BUFFER_SIZE) {
				write_logi("WARNING, expected to read from file: ", BUFFER_SIZE);
				write_logi("WARNING, read from file only: ", *num_read);
				write_logi("WARNING, current_file_pos: ", current_file_pos);
				write_logi("WARNING, lseek pos: ", lseek(fd_file, 0, SEEK_CUR));
				struct stat tstat;
				fstat(fd_file, &tstat);
				write_logi("WARNING, file size: ", tstat.st_size);
			}
			if (playspeed < 0) {
				lseek(fd_file, -*num_read, SEEK_CUR);
			}
			if (*num_read == 0) {
				end_of_file = 1;
			} else {
				playrespeed(&buffers[buf_head % NUM_BUFFERS]);
				buf_head++;
			}
		}
	}

	return *num_read;
}

int CSoundIO::write_file(void) {
	int num_written = 0;
	int count;
	char *pbuf;

	/* Is buffer dirty? Should we write something? */
	if (buf_tail != buf_head) {
		count = buffers[buf_tail % NUM_BUFFERS].count;
		pbuf = (char *)&buffers[buf_tail % NUM_BUFFERS].data;
		if (cut_in_latency) {
			cut_in_latency -= count;
			if (cut_in_latency < 0) {
				pbuf += abs(cut_in_latency);
				count += cut_in_latency;
				cut_in_latency = 0;
			} else {
				count = 0;
			}
		}
		if (count > 0) {
			if (make_ramp_attack) {
				make_ramp_attack = 0;
				ramp_attack(buf_tail);
			}
			num_written = write(fd_file, pbuf, count );
		}
		buf_tail++;
		current_file_pos += num_written;
	}

	return 0;
}

int CSoundIO::findmaxvolume(BUF_TYPE *buf, int buffer_count) {
	BUF_TYPE maxvol = 0;
	int i = 0;

	if (buffer_count > 0) {
		while (i < buffer_count/sizeof(BUF_TYPE)) {
			if (maxvol < abs(*buf)) {
				maxvol = (*buf < 0 ? abs(1+*buf) : *buf);
			}
			buf++;
			i++;
		}
	}

	return maxvol;
}

int CSoundIO::ramp_attack(int buf_num) {
	int temp;
	int numerator, denominator;
	BUF_TYPE *p;

	/* ramp up at start of sound */
	/* ramp up is on 'head' */
	denominator = buffers[buf_num % NUM_BUFFERS].count / sizeof(BUF_TYPE);
	if (denominator > 0) {
		p = &buffers[buf_num % NUM_BUFFERS].data[0];
		numerator = 1;
		while (numerator < denominator) {
			*p = ((int)*p * numerator) / denominator;
			*p++;
			numerator++;
		}
	}

	/* ramp down previous block */
	if (current_file_pos >= BUFFER_SIZE) {
		/* yes, we are not at start of file, seek back */
		if (lseek(fd_file, current_file_pos-BUFFER_SIZE, SEEK_SET) >= 0) {
			denominator = read(fd_file, &temp_buf, BUFFER_SIZE) / sizeof(BUF_TYPE);
			if (denominator > 0) {
				p = (BUF_TYPE *)&temp_buf;
				/*
				numerator = denominator;
				while (numerator) {
					*p = ((int)*p * numerator) / denominator;
					p++;
					numerator--;
				}
				*/
				/* -------- */
				p += denominator-1;
				while (*p < 0 && p > (BUF_TYPE *)&temp_buf) {
					*p = 0;
					p--;
				}
				while (*p > 0 && p > (BUF_TYPE *)&temp_buf) {
					*p = 0;
					p--;
				}
				write_logi("Cut off prev: ", p-(BUF_TYPE *)&temp_buf);
				/* -------- */
				lseek(fd_file, current_file_pos-BUFFER_SIZE, SEEK_SET);
				write(fd_file, &temp_buf, BUFFER_SIZE);
			}
		}
	}

	return 0;
}

int CSoundIO::write_log(char *text, char *extra) {
	struct timeval now;
	struct tm *tmdata;
	char textdata[200];

	gettimeofday(&now, NULL);
	tmdata = gmtime(&now.tv_sec);
	snprintf((char *)textdata, sizeof(textdata)-1,
			"%04d-%02d-%02d %02d:%02d:%02d.%03d ",
			tmdata->tm_year+1900,
			tmdata->tm_mon+1,
			tmdata->tm_mday,
			tmdata->tm_hour,
			tmdata->tm_min,
			tmdata->tm_sec,
			now.tv_usec / 1000);
	strncat((char *)textdata, text, sizeof(textdata)-1);
	if (extra) {
		strncat((char *)textdata, extra, sizeof(textdata)-1);
	}
	strncat((char *)textdata, "\n", sizeof(textdata)-1);
	write(fd_log, textdata, strlen(textdata));

	return 0;
}

int CSoundIO::write_logi(char *text, long int value) {
	char textdata[100];

	snprintf((char *)textdata, sizeof(textdata)-1, "%ld", value);

	return write_log(text, (char *)&textdata);
}

BUF_TYPE CSoundIO::peakadd(BUF_TYPE volume) {
	int i;
	BUF_TYPE max = volume;

	peaksamples[++peakcount % (sizeof(peaksamples) / sizeof(peaksamples[0]))] = volume;
	for (i=0; i<(sizeof(peaksamples) / sizeof(peaksamples[0])); i++) {
		if (max < peaksamples[i]) {
			max = peaksamples[i];
		}
	}
	if (opt_verbose) {
		printf("\e[0;30;47m\e[2;2HPeak: %7d", max);
	}
	peakvolume /= 5;
	peakvolume *= 3;
	if (peakvolume < max) {
		peakvolume = max;
	}

	return peakvolume;
}

int CSoundIO::correctplayspeed(int ps) {

	if (ps == 0) {
		ps = 1;	/* on action, playspeed can not be 0 */
	} else {
		/*
		if (ps > 4) {
			ps = 4;
		} else {
			if (ps < -4) {
				ps = -4;
			}
		}
		*/
	}
	/*
	switch (ps) {
		case -3:
			ps = -4;
			break;
		case 3:
			ps = 4;
			break;
	}
	*/
	if (opt_verbose) {
		write_logi("playspeed: ", ps);
	}

	return ps;
}

int CSoundIO::stoprecording(int doit) {

	if (doit) {
		int exit_count = CUT_OUT_LATENCY;
		while (exit_count--) { /* latency */
			read_dsp();
		}
		while (buf_head != buf_tail) {
			write_file();
		}
		ramp_attack(buf_head); /* trouble: if records starts now, we will ramp twice */
		/* Trunk? */
	}

	return 0;
}

tape_action CSoundIO::doaction(tape_action action) {
	int ioctl_value = 0;

	if (opt_verbose) {
		printf("\e[0;30;47m\e[1;2HFile: %s Heart beat:%5d beet2:%5d free:%3d used:%3d %3d %3d ps:%2d ",
				filename,
			(beat++) % 10000, beat2 % 10000, 
			buffree(buf_head,buf_tail), bufused(buf_head,buf_tail),
			buf_head % NUM_BUFFERS, buf_tail % NUM_BUFFERS, playspeed );
	}

	switch (action) {
		case tape_quit:
		case tape_stop:
			stoprecording( mode == mode_record );
			buf_tail = buf_head;
			playspeed = 0;
			if (mode == mode_forward) {
				write_logi("Spool back from: ", current_file_pos);
				/* Spool buffer backwards */
				current_file_pos -= opt_spoolback * sizeof(BUF_TYPE);
				if (current_file_pos < 0) {
					current_file_pos = 0;
				}
			}
			write_logi("Stop at file pos: ", current_file_pos);
			lseek(fd_file, current_file_pos + sizeof(wavheader), SEEK_SET);
			/* Stop playing NOW! */
			//ioctl(fd_dsp, SNDCTL_DSP_SYNC, &ioctl_value);
			mixer->set(stop);
			mode = mode_stop;
			break;
		case tape_record:
			end_of_file = 0;
			if (mode != mode_record) {
				/* Stop playing NOW! */
				//ioctl(fd_dsp, SNDCTL_DSP_SYNC, &ioctl_value);
				/* Flush buffers */
				buf_tail = buf_head;
				if (mode == mode_forward) {
					write_logi("Spool back from: ", current_file_pos);
					/* Spool buffer backwards */
					current_file_pos -= opt_spoolback * sizeof(BUF_TYPE);
					if (current_file_pos < 0) {
						current_file_pos = 0;
					}
				}
				/* Delete rest of file */
				if (0!=ftruncate(fd_file, current_file_pos + sizeof(wavheader))) {
					write_log("Error, ftruncate: ", strerror(errno));
				}
				write_logi("Start rec at file pos: ", current_file_pos);
				/* Seek to end of file (it is truncated) */
				lseek(fd_file, 0, SEEK_END);
				make_ramp_attack = 1;
				if (mode == mode_forward) {
					write_logi("cutplay: ", opt_cutplay);
					cut_in_latency = sizeof(BUF_TYPE) * opt_cutplay;
				} else {
					write_logi("cutin: ", opt_cutin);
					cut_in_latency = sizeof(BUF_TYPE) * opt_cutin;
				}
			}
			did_record = 1;
			mixer->set(record);
			mode = mode_record;
			break;
		case tape_play:
		case tape_forward_1:
			if (playspeed < 0) {
				playspeed = 0;
			}
			playspeed = correctplayspeed(playspeed + 1);
			stoprecording( mode == mode_record );
			end_of_file = 0;
			lseek(fd_file, current_file_pos + sizeof(wavheader), SEEK_SET);
			buf_tail = buf_head;
			read_file();
			mixer->set(playback);
			mode = mode_forward;
			break;
		case tape_edit:
			playspeed = 1-opt_revspeed;
		case tape_rewind_1:
			if (playspeed > 0) {
				playspeed = 0;
			}
			playspeed = correctplayspeed(playspeed - 1);
			stoprecording( mode == mode_record );
			end_of_file = 0;
			lseek(fd_file, current_file_pos + sizeof(wavheader), SEEK_SET);
			buf_tail = buf_head;
			read_file();
			mixer->set(playback);
			mode = mode_rewind;
			break;
		case tape_track_next:
			stoprecording( mode == mode_record );
			if (lseek(fd_file, 0, SEEK_END) > sizeof(wavheader)) {
				fd_file = opennewfilename(filename);
			} else {
				current_file_pos = 0;
				lseek(fd_file, current_file_pos + sizeof(wavheader), SEEK_SET);
			}
			if (mode == mode_record) {
				terminal->setbuttonon(but_record);
				did_record = 1;
			}
			//mixer->set(stop);
			break;
		case tape_track_prev:
			stoprecording( mode == mode_record );
			fd_file = openprevfilename(fd_file);
			mixer->set(stop);
			mode = mode_stop;
			break;
		case tape_menu:
			run_menu();
			break;
	}

	return action;
}

tape_mode CSoundIO::domode(fd_set *rd_set, fd_set *wr_set) {
	switch (mode) {
		case mode_record:
			if (FD_ISSET(fd_dsp, rd_set))
				read_dsp();
			if (FD_ISSET(fd_file, wr_set))
				write_file();
			break;
		case mode_stop:
			if (FD_ISSET(fd_dsp, rd_set))
				read_dsp();
			buf_tail = buf_head;
			break;
		case mode_play:
		case mode_forward:
		case mode_rewind:
			if (FD_ISSET(fd_dsp, wr_set)) {
				write_dsp();
			}
			read_file();
			break;
	}

	return mode;
}

int CSoundIO::fd_set_rw(fd_set *rd_set, fd_set *wr_set) {
	switch (mode) {
		case mode_stop:
			FD_SET(fd_dsp, rd_set);
			FD_CLR(fd_dsp, wr_set);
			FD_CLR(fd_file, rd_set);
			FD_CLR(fd_file, wr_set);
			break;
		case mode_record:
			FD_SET(fd_dsp, rd_set);
			FD_CLR(fd_dsp, wr_set);
			FD_CLR(fd_file, rd_set);
			/* if buffer dirty */
			if (buf_head == buf_tail) {
				FD_CLR(fd_file, wr_set);
			} else {
				FD_SET(fd_file, wr_set);
			}
			break;
		case mode_play:
		case mode_forward:
		case mode_rewind:
			FD_CLR(fd_dsp, rd_set);
			FD_SET(fd_dsp, wr_set);
			FD_CLR(fd_file, rd_set);
			FD_CLR(fd_file, wr_set);
			break;
		default:
			mode = mode_stop;
	}

	return 0;
}

long CSoundIO::sec10(void) {
	return current_file_pos / BYTES_PER_SEC10;
}

/* -------- CSoundIO --------- */

tape_key lastcmd = key_none;

#define HOUR_S10 10*60*60L
#define MIN_S10  10*60L
#define SEC_S10  10L
#define MAX_SEC10 (HOUR_S10*100-1)

int CSoundIO::displaysecond10(long sec10) {
	int i;
	off_t sec_index;
	char newsegmentbuf[NUM_SEGMENTS+1];

	sec_index = (counter_size - counter_index) / (BYTES_PER_SEC10);
	if (sec10 + sec_index >= 0) {
		sec10 += sec_index;
	} else {
		sec10 += (counter_size) / (BYTES_PER_SEC10);
	}
	if (sec10 < 0) {
		sec10 = 0;
	}
	if (sec10 > MAX_SEC10) {
		sec10 %= MAX_SEC10+1;
	}
	if (lastsecond != sec10) {
		snprintf(newsegmentbuf, sizeof(newsegmentbuf), "%02d%02d%02d%d",
			(sec10 % 3600000L) / 36000, /* 0..99 hours */
			(sec10 %   36000L) /   600, /* 0..59 minutes */
			(sec10 %     600L) /    10, /* 0..59 seconds */
			(sec10 %      10L) );       /* 0..9 1/10 seconds */
		if (opt_verbose) {
			write_log("Disp.sec: ", newsegmentbuf );
			write_logi("current_file_pos: ", current_file_pos);
			write_logi("File pos lseek(): ", lseek(fd_file, 0, SEEK_CUR));
		}
		i = 1;
		/* if timer overflow > 59 minutes */
		if (sec10 < (36000) || ((sec10 / 10) % 4 == 0)) {
			i += 2;
			newsegmentbuf[0] = filename[NFILENAMEDIGIT-2];
			newsegmentbuf[1] = filename[NFILENAMEDIGIT-1];
		}
		while (i<6) {
			/* add a decimal dot, pos: 1, 3, 5 */
			newsegmentbuf[i] |= 0x80;
			i += 2;
		}
		if (opt_lf30hz) {
			newsegmentbuf[6] |= 0x80;
		}
		terminal->showsegmenttext((char *)newsegmentbuf);
		lastsecond = sec10;
	}
	return sec10;
}

tape_action CSoundIO::getaction(fd_set *rd_set) {
	int i;
	tape_key key = key_none;
	tape_action action = tape_noaction;

	key = terminal->getcommand(rd_set, key);
	i = 0;
	while (i<SIZE_ACTIONBUTTON && action == tape_noaction) {
		if (key == actionbutton[i].key) {
			action = actionbutton[i].action;
			terminal->setbuttonon(actionbutton[i].button);
		}
		i++;
	}

	return action;
}

tape_action CSoundIO::takeaction(fd_set *rd_set, fd_set *wr_set) {
	tape_action action;

	action = getaction(rd_set);
	if (action != tape_noaction) {
		write_log("Action: ", actionname(action));
	}
	action = doaction(action);
	domode(rd_set, wr_set);
	if (mode != mode_stop) {
		displaysecond10(sec10());
	}
	if (mode == mode_stop && lastmode != mode_stop) {
		terminal->setbuttonon(but_stop);
	}
	lastmode = mode;
	if (action == tape_track_next || action == tape_track_prev) {
		lastsecond = -1;
		displaysecond10(sec10());
	}

	return action;
}

void CSoundIO::reportterms(CConsole *temp) {

	while (temp) {
		if (temp->nameofdevice)
			write_log("Console open: ", temp->nameofdevice);
		else
			write_log("Console open failed, no name", NULL);
		temp = temp->next;
	}

}

int CSoundIO::dispset(int col, fd_set *rd_set, fd_set *wr_set) {
	int i;
	char *s;

	if (opt_verbose) {
		printf("\e[11;%dH\e[0mRD: ", col);
		printf("file %d, ", FD_ISSET(fd_file, rd_set));
		printf("dsp %d, ", FD_ISSET(fd_dsp, rd_set));
		printf("\e[12;%dHWR: ", col);
		printf("file %d, ", FD_ISSET(fd_file, wr_set));
		printf("dsp %d, ", FD_ISSET(fd_dsp, wr_set));
		switch (mode) {
			case mode_stop:
				s = "stop";
				break;
			case mode_play:
				s = "play";
				break;
			case mode_record:
				s = "record";
				break;
			case mode_forward:
				s = "forward";
				break;
			case mode_rewind:
				s = "rewind";
				break;
		}
		printf("\e[13H Mode = %s   ", s);
	}
}

void CSoundIO::run_menu(void) {
	tape_action action = tape_stop;
	int res = 0;
	CMenu *menu;
	int tidx = 0;
	int hselect = 0;
	fd_set rd_set;
	struct timeval tv;
	unsigned int second10 = 0;
	char textbuf[256];
	char *tp = (char *)&textbuf;
	int scroll_on = 0;
	int bail_out = 0;

	counter_size = sumfilesize(atoi(filename)) + current_file_pos;
	closewavfile();
	menu = new CMenu(filename);
	FD_ZERO(&rd_set);
	strncpy(textbuf, menu->current_text[tidx], sizeof(textbuf));

	while (!bail_out) {
		terminal->showsegmenttext(tp);
		terminal->flush();
		terminal->fd_set_read(&rd_set);
		hselect = max(hselect, terminal->higest_fd);
		tv.tv_sec = 0;
		tv.tv_usec = 200000;
		res = select(hselect+1, &rd_set, NULL, NULL, &tv);
		if (res < 0) {
			printf("Select error %d at line %d\n", res, __LINE__ );
		} else {
			if (res) {
				action = getaction(&rd_set);
				if (action != tape_noaction) {
					write_log("Action: ", actionname(action));
					write_log("Menu on display 1: ", menu->current_text[0]);
					bail_out = menu->do_action(action);
					switch (bail_out) {
						case 1:
							write_log("Menu on display 2: ", menu->current_text[0]);
							break;
						case 2:
							write_log("Execute command: ", menu->current_text[0]);
							break;
					}
					tidx = 0;
					strncpy(textbuf, menu->current_text[tidx], sizeof(textbuf));
					scroll_on = 0;
					tp = textbuf;
					second10 = 0;
				}
			} else {
				/* time out */
				second10++;
				/* is scrolling active? */
				if (scroll_on) {
					tp++;
					scroll_on = strlen(tp);
					if (!scroll_on) {
						second10 = 9;
					}
				} else {
					if (!(second10 % 10)) {
						tp = textbuf;
						tidx = (tidx + 1) % menu->current_tcount;
						if (strlen(menu->current_text[tidx]) <= NUM_SEGMENTS) {
							strncpy(textbuf, menu->current_text[tidx], sizeof(textbuf));
							scroll_on = 0;
						} else {
							memset(textbuf, ' ', NUM_SEGMENTS);
							strncpy((char *)&textbuf[NUM_SEGMENTS], menu->current_text[tidx],
									sizeof(textbuf)-NUM_SEGMENTS);
							scroll_on = strlen(tp);
						}
					}
				}
			}
		}
	}
	write_log("Result command from menu: ", menu->current_text[0] );
	fd_file = openexistingfilename(filename);
	displaysecond10(sec10());
	terminal->setbuttonon(but_stop);

	delete menu;
}

long usecdelay(struct timeval *timemark) {
	struct timeval now;

	gettimeofday(&now, NULL);

	return now.tv_usec % 100000;
}

int main(int argc, char **argv) {
	int hselect;
	fd_set rd_set;
	fd_set wr_set;
	struct timeval tv;
	CLogVolume lvol;
	tape_action action = tape_stop;
	int selector;
	GetOption getoption(argc, argv, help_long_options);

	if (getoption.status == 1) {
		cout << "Try `" << argv[0] << " --help' for more information." << endl;
		return 0;
	}

	if (opt_version) {
		cout << PACKAGE_NAME << " version " << VERSION << endl;
		return 0;
	}

	/* Load consoles */
	LinkConsole term;

#ifdef BLIND
	if (opt_blind)
		term.addconsole(new TermConsole());
	else
		term.addconsole(new BlindConsole());
#else
	if (opt_blind)
		term.addconsole(new BlindConsole());
	else
		term.addconsole(new TermConsole());
#endif

	if (opt_tapebox1)
		term.addconsole(new TapeboxConsole(opt_tapebox1));
	if (opt_tapebox2)
		term.addconsole(new TapeboxConsole(opt_tapebox2));
	if (opt_devmouse)
		term.addconsole(new MouseConsole(opt_devmouse));
	if (opt_devjoy)
		term.addconsole(new JoyConsole(opt_devjoy));

	if (opt_help) {
		cout << "Command line options for " PACKAGE_NAME " " VERSION "." << endl;
 		cout << "Copyright  " YEAR " by Schou Industries ApS." << endl;
		cout << "Build date: " << ISODATE << "." << endl;
		cout << "Max recording time: " << MAX_REC_TIME << endl << endl;
		getoption.help(fileno(stdout));
		cout << endl;
		CMenu *menu = new CMenu("");
		menu->help(fileno(stdout));
		cout << endl;
		cout << "Send bug report to: <" << PACKAGE_BUGREPORT << ">" << endl;
		cout << "http://schou.dk/ http://www.speltron.dk/" << endl;
		if (opt_verbose)
			term.help(fileno(stdout));
		return 0;
	}

	term.redraw();

	CSoundIO snd(&term);
	snd.openfiles(opt_devdsp);
	hselect = 0;
	hselect = max(hselect, snd.fd_dsp);
	hselect = max(hselect, snd.fd_file);

	snd.reportterms(term.head);

	term.setvolume(60,60); term.flush();
	if (!opt_command) {
		term.showsegmenttext((char *)"DSKFREE"); term.flush(); usleep(400000);
		term.showsegmenttext(snd.diskfreetext()); term.flush(); usleep(400000);
		term.showsegmenttext(snd.diskfreetime); term.flush(); usleep(400000);
	}
	if (opt_verbose) {
		term.showsegmenttext((char *)"VERBOSE");
	} else {
		snd.displaysecond10(0);
	}

	snd.write_logi("spoolback samples: ", opt_spoolback);

	if (opt_debug) {
		switch (opt_debug[0]) {
			case '1':
				term.setvolume(0,0); term.flush(); sleep(1);
				int v = 0;
				while (v<=60) {
					snd.displaysecond10(v);
					term.setvolume(0,v);
					term.flush();
					usleep(100000);
					v++;
				}
				v = 0;
				while (v<=60) {
					snd.displaysecond10(v);
					term.setvolume(v,0);
					term.flush();
					usleep(100000);
					v++;
				}
				term.setvolume(60,0); sleep(5);
				term.setvolume(0,0);
				break;
		}
	}

	FD_ZERO(&rd_set);
	FD_ZERO(&wr_set);

	/* XXX Fejl ved Lydavisen's ene Zitech med Audigy-kort */
	snd.write_log("Init go record.", 0);
	snd.doaction(tape_record);
	snd.write_log("Init go stop.", 0);
	snd.doaction(tape_stop);

	if (opt_command) {
		if (!strcmp(opt_command,"record")) {
			snd.doaction(tape_record);
			term.setbuttonon(but_record);
		}
	}

	snd.write_log("Starting main loop.", 0);
	while (action != tape_quit) {
		term.flush();
		term.fd_set_read(&rd_set);
		snd.fd_set_rw(&rd_set, &wr_set);
		hselect = max(hselect, term.higest_fd);
		tv.tv_sec = 0;
		tv.tv_usec = usecdelay(&snd.timemark);
		snd.dispset(1,&rd_set, &wr_set);
		selector = select(hselect+1, &rd_set, &wr_set, NULL, &tv);
		if (selector) {
			action = snd.takeaction(&rd_set, &wr_set);
			beat2++;
		} else {
			term.setvolume(	lvol.logvol(snd.maxvolume), 
					lvol.logvol(snd.peakadd(snd.maxvolume)));
			if (reopen_sound_count) {
				if (--reopen_sound_count == 0) {
					snd.write_log("Autimatically re-open sound", NULL);
					snd.mixer->set(stop);
				}
			}
			beat++;
		}
		snd.dispset(40,&rd_set, &wr_set);
	}
	snd.write_log("Exit main loop.", 0);

	cout << "\e[23H\e[0m" << endl;
#ifdef PACKAGE_NAME
#  ifdef VERSION
	cout << "Thanks for using " << PACKAGE_NAME << " version " << VERSION << endl;;
#  endif
#endif
#ifdef PACKAGE_BUGREPORT
	cout << "Send bug-report to: <" << PACKAGE_BUGREPORT << ">" << endl;
#endif
	return 0;
}

