/*
 * Speak Recorder
 * Copyright  2004 by Schou Industries ApS.
 */

/*
 * --------------------------------------------------------------
 * http://www.bluesock.org/~willg/dev/ansi.html
 */

#include <iostream>
#include <unistd.h>
#include <termios.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "font_8x8.inc"
#include "termconsole.h"

/* ANSI codes */

#define BLACK  30
#define RED    31
#define GREEN  32
#define YELLOW 33
#define BLUE   34
#define PURPEL 35
#define BROWN  36
#define GRAY   37

#define BGBLACK  40
#define BGRED    41
#define BGGREEN  42
#define BGYELLOW 43
#define BGBLUE   44
#define BGPURPEL 45
#define BGBROWN  46
#define BGGRAY   47

/* Size and position of volume bar */

#define VWIDTH 61 /* 0..60 */
#define BARSPLIT 6

#define VBASEX 10
#define VBASEY 15
#define VHEIGHT 1  

#define X_GREEN 0
#define X_YELLOW 48
#define X_RED VWIDTH-4

/* ------- Buttons -------- */

#define COUNT_BUTTON 7
#define BUTTON_COLOR_OFF "\e[0;30;47m"
#define BUTTON_COLOR_ON  "\e[0;31;44m"
#define BUTTON_WIDTH 10

#define BBASEY (VBASEY+VHEIGHT+3)
#define BBASEX 7

#define BUT_RECORD  0
#define BUT_REWIND  1
#define BUT_FORWARD 2
#define BUT_STOP    3
#define BUT_PLAY    4
#define BUT_EDIT    5
#define BUT_TAPE    6

/* ------- Segments -------- */

#define SWIDTH 8
#define SHEIGHT 8
#define SXPAD 1
#define SBASEX 7
#define SBASEY 3

#define HOUR_S10 10*60*60L
#define MIN_S10  10*60L
#define SEC_S10  10L
#define MAX_SEC10 HOUR_S10*100-1

TermConsole::TermConsole(void) {
	struct termios termattr;

	nameofdevice = "Terminal, full screen color TUI";
	fd = fileno(stdin);
	if (0 > tcgetattr(fd, &save_termios)) {
		fprintf(stderr,"Error save_termios\n");
	}
	memcpy(&termattr, &save_termios, sizeof(termattr));
	termattr.c_lflag &= ~(ICANON | ECHO);
	termattr.c_cc[VMIN]  = 1;
	termattr.c_cc[VTIME] = 0;
	if (0 > tcsetattr(fd, TCSANOW, &termattr)) {
		fprintf(stderr,"Error set termattr\n");
	}
}

TermConsole::~TermConsole(void) {
	cout << "\e[0m";
	tcsetattr(fd, TCSANOW, &save_termios);
};

int TermConsole::flush() {
	cout << "\e[0m\e[H";
	fflush(stdout);
}

struct {
	int color;
	char *name;
	button_name but;
} buttons[] = {
	{ color:40, name:"Record", but:but_record     },
	{ color:40, name:" Edit ", but:but_edit       },
	{ color:40, name:" Stop ", but:but_stop       },
	{ color:40, name:" Play ", but:but_play       },
	{ color:40, name:" Menu ", but:but_menu       },
	{ color:40, name:" preV ", but:but_track_prev },
	{ color:40, name:" nexT ", but:but_track_next }
};

#define SIZE_BUTTONS (sizeof(buttons) / sizeof(buttons[0]))

int showbutton( int num, int on ) {
	int posx = num * BUTTON_WIDTH + BBASEX;

	printf("\e[%d;%dH\e[1;37;%dm        ", BBASEY, posx,
			on ? 41 : 40 );
	printf("\e[%dD\e[B %s ", BUTTON_WIDTH - 2, buttons[num].name);
	printf("\e[%dD\e[B        ", BUTTON_WIDTH - 2);

	return 0;
}

int TermConsole::setbuttonon(button_name name) {
	int i;

	for (i=0; i<SIZE_BUTTONS; i++) {
		if (buttons[i].but == buttonon) {
			showbutton(i, 0);
		}
		if (buttons[i].but == name) {
			showbutton(i, 1);
		}
	}
	buttonon = name;

	return 0;
}

int TermConsole::redraw() {
	int i,n;

	cout << "\e[0;" << GRAY << ";" << BGGRAY << "m\e[2J\e[H";
	cout << "\e[" << BLACK << ";" << BGGRAY << "m";
	cout << "\e[" << VBASEY -1 << ";" << VBASEX -2 << "H";
	for (i=(VWIDTH-1) / BARSPLIT; i>=0; i--) {
		printf("%3d", i*-BARSPLIT);
		n = BARSPLIT-3;
		while (n--) putchar(' ');
	}
	setvolume(VWIDTH-1, 0);
	setvolume(0,0);

	for (i=0; i<COUNT_BUTTON; i++) {
		showbutton( i, 0);
	}

	return 0;
}

int TermConsole::fd_set_read(fd_set *read_set) {

	FD_SET(fd, read_set);

	return fd;
}

int TermConsole::fd_set_write(fd_set *write_set) {

	return 0;
}

int TermConsole::putcha( char c ) {

	if (c != lastcolor) {
		printf("\e[4%cm", lastcolor = c ? '0' : '7' );
	}
	putchar(' ');
	if (SXPAD > 1) {
		putchar(' ');
	}

	return 0;
}

unsigned char TermConsole::bignum( unsigned char n ) {
	int j, i;

	int doton = n > 0x7F;
	n &= 0x7F;
	for (j=0; j<SHEIGHT; j++) {
		i = 0x80;
		do {
			if (i<2 && j==SHEIGHT-1 && doton)
				putcha(1);
			else
				putcha(fontdata_8x8[ n*SHEIGHT + j ] & i);
			i /= 2;
		} while (i);
		printf("\e[%dD\e[B", SWIDTH*SXPAD );
	}
	return n;
}

int TermConsole::showsegment( int num, unsigned char value ) {

	lastcolor = -1;
	if (num < 0 || num > NUM_SEGMENTS) {
		return 0;
	}
	printf("\e[%d;%dH", SBASEY, num*2 + num*SWIDTH*SXPAD + SBASEX + (num > 1 ? 3 : -2) );
	bignum( value );

	return value;
}

struct {
	char key;
	tape_key cmd;
	char *help;
} term_key[] = {
	{ 's', key_stop, "Stop" },
	{ 'p', key_play, "Play" },
	{ 'r', key_record, "Record" },
	{ 't', key_track_next, "Next track" },
	{ 'v', key_track_prev, "Previous track" },
	{ 'w', key_rewind_inc, "Rewind" },
	{ 'W', key_rewind_dec, NULL },
	{ 'f', key_forward_inc, "Forward" },
	{ 'F', key_forward_dec, NULL },
	{ 'e', key_edit, "Edit" },
	{ 'm', key_menu, "Menu" },
	{ 'q', key_quit, "Quit" }
};

#define SIZE_TERM_KEY (sizeof(term_key) / sizeof(term_key[0]))

tape_key TermConsole::getcommand(fd_set *read_set, tape_key lastcmd) {
	char ch;
	int i = 0;

	if (FD_ISSET(fd, read_set)) {
		ch = getc(stdin);
		while (i<SIZE_TERM_KEY && term_key[i].key != ch)
			i++;
		if (i>=SIZE_TERM_KEY) {
			return key_none;
		} else {
			return term_key[i].cmd;
		}
	}

	return key_none;
}

int TermConsole::help(int outdev) {
	int i = 0;
	
	dprintf(outdev, "Help for Text User Interface (TUI).\n");
	dprintf(outdev, "Keys:\n");
	while (i<SIZE_TERM_KEY) {
		if (term_key[i].help) {
			dprintf(outdev, "\t%c  %s\n", term_key[i].key, term_key[i].help);
		}
		i++;
	}

	return 0;
}

/* ------------------ Volume ----------------- */

typedef enum {
	pos_undef,
	pos_black,
	pos_green,
	pos_yellow,
	pos_red
} pos_color;

pos_color lastbarcolor = pos_undef;

int barsymbol(int pos, int coloron) {
	pos_color newcolor;

	if (coloron) {
		if (pos < X_YELLOW)
			newcolor = pos_green;
		else if (pos < X_RED)
			newcolor = pos_yellow;
		else
			newcolor = pos_red;
	} else {
		newcolor = pos_black;
	}
	if (lastbarcolor != newcolor) {
		switch (newcolor) {
			case pos_black:
				cout << "\e[" << BGBLACK << "m";
				break;
			case pos_green:
				cout << "\e[" << BGGREEN << "m";
				break;
			case pos_yellow:
				cout << "\e[" << BGYELLOW << "m";
				break;
			case pos_red:
				cout << "\e[" << BGRED << "m";
				break;
		}
		lastbarcolor = newcolor;
	}
	switch (pos % BARSPLIT) {
		case 0:
			putchar('|');
			break;
		case BARSPLIT/2:
			putchar('+');
			break;
		default:
			putchar('-');
	}

	return pos;
}

int TermConsole::setvolume(int newvol, int peakvol) {
	int i, k;

	cout << "\e[1;" << GRAY << "m";
	lastbarcolor = pos_undef;
	for (k=0; k<VHEIGHT; k++) {
		cout << "\e[" << VBASEY+k << ";" << VBASEX << "H";
		i = 0;
		while (i < newvol)
			barsymbol(i++, 1);
		if (i == newvol && i > 0)
			barsymbol(i++, 1);
		while (i < peakvol)
			barsymbol(i++, 0);
		if (i == peakvol && i > 0)
			barsymbol(i++, 1);
		while (i < currentpeak+1)
			barsymbol(i++, 0);
	}
	currentvolume = newvol;
	currentpeak = peakvol;

	printf("\e[30;47m\e[%d;%dH%3ddB", VBASEY+VHEIGHT, VBASEX, peakvol-60);

	return currentvolume;
}

